@@ -166,6 +166,7 @@ d_fpclassify='undef'
 d_fpclassl='undef'
 d_fpgetround='undef'
 d_fpos64_t='undef'
+d_freelocale='undef'
 d_frexpl='undef'
 d_fs_data_s='undef'
 d_fseeko='undef'
@@ -307,6 +308,7 @@ d_mbtowc='undef'
 d_memchr='define'
 d_memcmp='define'
 d_memcpy='define'
+d_memmem='undef'
 d_memmove='define'
 d_memset='define'
 d_mkdir='define'
@@ -338,6 +340,7 @@ d_mymalloc='undef'
 d_nan='undef'
 d_ndbm_h_uses_prototypes='undef'
 d_nearbyint='undef'
+d_newlocale='undef'
 d_nextafter='undef'
 d_nexttoward='undef'
 d_nice='undef'
@@ -537,6 +540,7 @@ d_uname='undef'
 d_union_semun='undef'
 d_unordered='undef'
 d_unsetenv='undef'
+d_uselocale='undef'
 d_usleep='define'
 d_usleepproto='undef'
 d_ustat='undef'
